/* 18_Telecomando
mainRX.c  - PIC del ricevitore
Paolo Reg 26/4/2011
spunti: 
	http://dev.sarasini.it/?p=224  Telecomando IR con Pic
modulo ricevitore Vishay TSOP34838 (38 kHz)

sezione RX:
out del ricevitore su RC0
led su RC1 lampeggia a frequenza overflow di TMR0
led su RC2 replica l'out del ricevitore
display su PORTD
*/

#pragma config FOSC=HS,LVP=OFF,WDT=OFF
#pragma config PWRT = ON,PBADEN=OFF
#pragma config CCP2MX = ON		//modulo CCP2 su RC1

// -- VARIABILI GLOBALI --
volatile unsigned int iOVF=0,iOVFd=0;	// iOVF contatore di overflow di TMR1

// -- INCLUDE --
#include <p18f4550.h>
#include <stdio.h>
#include <timers.h>
// usa Delay1KTCYx(n) ritardo (0,2 x n) ms, Delay10KTCYx(n) ritardo (2 x n) ms
#include <delays.h>	
#include <usart.h>

// libreria LCD
void delays_ms(unsigned long);	//richiamata in LCD_44780.c e keyb.c
#define LCD_DEFAULT
#include "LCD_44780.c"
//
// libreria del telecomando
// in telec.h ci sono le specifiche del telecomando e le costanti
// l'unica funz. pubblica  AscoltaRicev() ; qui si usa la costante SIGLATELEC
#define RECEIV PORTCbits.RC0	// ingresso dal modulo receiver (attivo basso)	
#include "telec.c"
//		
// -- DEFINE --
#define LED1 LATCbits.LATC1		// 
#define LED2 LATCbits.LATC2		// replica RECEIV

// -- ISR - routines interruzioni --
// prototipo funzione che gestisce l'interrupt ad alta priorit
void hInt_Event(void);

// l'interruzione fa saltare alla locazione 0x08, detta interrupt vector 
#pragma code h_int_vect=0x08
void h_int(void) {
	_asm GOTO hInt_Event _endasm
}
#pragma code

// dichiaro che la funzione di gestione interrupt si chiama hInt_Event
#pragma interrupt hInt_Event

// definizione della funzione di gestione int (ad alta priorit, modo compatibile)
void hInt_Event(void) {
	if (INTCONbits.TMR0IF == 1) {	//  avvenuto un overflow in TMR0
		INTCONbits.TMR0IF = 0;	// resetto l'IF di TMR0
		// conteggio overflow; ogni ovf vale 50us
		iOVF++;
		if (iOVF==900)	LED1=!LED1;		// fa lampeggiare LED1
		iOVFd++;
		WriteTimer0(5); // per avere DT=(255-5)*0.2= 50us
		LED2=RECEIV;		// replico l'out del receiver su LED2
	}
}
// -- fine ISR --	

void setup(void){
	//------------------- Impostazione porte
	ADCON1=0x0F;			// porte come digital IO
	
	//PORTC bit 6 e 7 per l'USART:RC6=TX=0, RC7=RX=1
	// RC0 input dal receiver; R1,RC2 led  
	LATC=0x00;
	TRISC = 0b10111001;
	
	// RD2..RD7 output per l'LCD
	LATD=0;
	TRISD = 0; 

	// Imposta TMR0
	// TMR0 a 8 bit, T1_PS=1, ovf ogni 255*0.2us =51us;
	OpenTimer0(TIMER_INT_ON & T0_SOURCE_INT & T0_PS_1_1 & T0_8BIT );
	
	// abilito le interruzioni
	RCONbits.IPEN=0;	// modalit compatibile PIC16
 	INTCONbits.GIE=1;	// abilito l'int globale
 	INTCONbits.PEIE=1;	// abilito l'int periferiche

		// Configura l'USART
	// 8 bit, 19200 bit/s, 1 bit stop, 0 bit parit
	OpenUSART(USART_TX_INT_OFF & USART_RX_INT_OFF & USART_ASYNCH_MODE & USART_EIGHT_BIT & USART_CONT_RX & USART_BRGH_HIGH, 64);
}

void main (){
	setup();		// impostazioni
	
  	printf("\r\n telec.");
  	printf(SIGLATELEC);
  	printf("\n");

	OpenLCD(20);	// inizializzazione LCD
 	WriteStringLCD ("telec.");
	WriteStringLCD (SIGLATELEC);

	while (1) {
		AscoltaRicev();

	}	// while(1)
	
}		//main

void delays_ms(unsigned long cntr) {
	// Delay1KTCYx(1);	  ritardo di 0,2 ms con clock 20 MHz
	for (;cntr>0;cntr--)
		Delay1KTCYx(5);
}