/* 18_Telecomando
mainRX_d.c  - PIC del ricevitore
modulo ricevitore Vishay TSOP34838 (38 kHz)

questa versione memorizza solo le durate utili (marks dei command)

sezione RX:
out del ricevitore su RC0
led su RD0 attivo basso ripete l'out del ricevitore
display su PORTD
*/

#pragma config FOSC=HS,LVP=OFF,WDT=OFF
#pragma config PWRT = ON,PBADEN=OFF
#pragma config CCP2MX = ON		//modulo CCP2 su RC1


// -- INCLUDE --
// specifiche del telecomando
#include "telec.h"

#include <p18f4550.h>
#include <stdio.h>
#include <timers.h>
// usa Delay1KTCYx(n) ritardo (0,2 x n) ms, Delay10KTCYx(n) ritardo (2 x n) ms
#include <delays.h>	
#include <usart.h>
	

void delays_ms(unsigned long);	//richiamata in LCD_44780.c e keyb.c
#define LCD_DEFAULT
#include "LCD_44780.c"

// -- DEFINE --
#define RECEIV PORTCbits.RC0	// ingresso dal modulo receiver (attivo basso)
#define LED1 LATCbits.LATC1		// led su RC1
#define LED2 LATCbits.LATC2		// replica RECEIV

#define NOCOMMAND 255
#define MAXN 70	// n.max durate impulsi da registrare=dimensione array durate[]		
#define DISP_TK 20000 	// ogni quanti 50us visualizzare i dati (1s)
#define MARK 0
#define SPACE 1 

// -- PROTOTIPI FUNZIONI --
//void ScriviSeq(unsigned char durate[],unsigned char StatiRec[],unsigned char idurate);
void ScriviSeq(unsigned char durate[],unsigned char idurate);
unsigned char DecodSeq(unsigned char durate[],unsigned char Commands[]);
unsigned char Confronta(unsigned char cosa,unsigned char concosa,unsigned char Tol);

// -- VARIABILI GLOBALI --
unsigned char fBtnTelEn=0;
volatile unsigned int iOVF=0,iOVFd=0;	// iOVF contatore di overflow di TMR1

// -- ISR - routines interruzioni --
// prototipo funzione che gestisce l'interrupt ad alta priorit
void hInt_Event(void);

// l'interruzione fa saltare alla locazione 0x08, detta interrupt vector 
#pragma code h_int_vect=0x08
void h_int(void) {
	_asm GOTO hInt_Event _endasm
}
#pragma code

// dichiaro che la funzione di gestione interrupt si chiama hInt_Event
#pragma interrupt hInt_Event

// definizione della funzione di gestione int (ad alta priorit, modo compatibile)
void hInt_Event(void) {
	if (INTCONbits.TMR0IF == 1) {	//  avvenuto un overflow in TMR0
		INTCONbits.TMR0IF = 0;	// resetto l'IF di TMR0
		// conteggio overflow; ogni ovf vale 50us
		iOVF++;
		if (iOVF==900)	LED1=!LED1;		// fa lampeggiare LED1
		iOVFd++;
		WriteTimer0(5); // per avere DT=(255-5)*0.2= 50us
		LED2=RECEIV;		// replico l'out del receiver su LED2
	}
}
// -- fine ISR --	


void setup(void){
	//------------------- Impostazione porte
	ADCON1=0x0F;			// porte come digital IO
	
	//PORTC bit 6 e 7 per l'USART:RC6=TX=0, RC7=RX=1
	// RC0 input dal receiver; R1,RC2 led  
	LATC=0x00;
	TRISC = 0b10111001;
	
	// RD2..RD7 output per l'LCD
	LATD=0;
	TRISD = 0; 

	// Imposta TMR0
	// TMR0 a 8 bit, T1_PS=1, ovf ogni 255*0.2us =51us;
	OpenTimer0(TIMER_INT_ON & T0_SOURCE_INT & T0_PS_1_1 & T0_8BIT );
	
	// abilito le interruzioni
	RCONbits.IPEN=0;	// modalit compatibile PIC16
 	INTCONbits.GIE=1;	// abilito l'int globale
 	INTCONbits.PEIE=1;	// abilito l'int periferiche

		// Configura l'USART
	// 8 bit, 19200 bit/s, 1 bit stop, 0 bit parit
	OpenUSART(USART_TX_INT_OFF & USART_RX_INT_OFF & USART_ASYNCH_MODE & USART_EIGHT_BIT & USART_CONT_RX & USART_BRGH_HIGH, 64);
}


void main (){
	unsigned int InizioI,FineI,DurataI;
	unsigned char durate[MAXN]={0};
//	unsigned char StatiRec[MAXN];		// devo risparmiare memoria!
	unsigned char i,idurate=0,fNuovaSeq,StatoReceiv,fGap=0;
	unsigned char idurC=0;	// indice durate dei mark del Command
	unsigned char TastoRil;

	// impostazioni
	setup();
	
  	printf("\r\n telec.");
  	printf(SIGLATELEC);
  	printf("\n");

	OpenLCD(20);	// inizializzazione LCD
 	WriteStringLCD ("telec.");
	WriteStringLCD (SIGLATELEC);

	fNuovaSeq=1;
	while (1) {
		// se  una nuova sequenza
		if(fNuovaSeq) {
			fGap=0;		// devo cercare un gap
			// azzero gli array durate e stati
			for(i=0;i<MAXN;i++) {
				durate[i]=0;	
			//	StatiRec[i]=2;	// 2 = stato non valido
			}
			idurate=0;		// indice nell'array durate
			idurC=0;		// indice durate registrate (Start + mark dei Commands)
			// aspetto un mark successivo a un gap (space di durata>IMPTOUT_TK)		
			// continuo a misurare gli space finch non trovo un gap
			while(!fGap){
				if(RECEIV==SPACE){		// sono in space
					iOVF=0;
					InizioI=iOVF;
					while(RECEIV==SPACE);	//finch non passa a mark
					DurataI=iOVF-InizioI;
					if(DurataI>IMPTOUT_TK) fGap=1;
				}
			}	//while(!fGap)
			// finalmente ho trovato un gap; ora sono all'inizio del primo mark
			StatoReceiv=RECEIV;	// mark  (space)
			fNuovaSeq=0;
		}	// if(fNuovaSeq)
		// continuo a misurare gli impulsi finch non trovo un altro gap;
		iOVF=0;		// azzero la base tempi 
		InizioI=iOVF;
		StatoReceiv=RECEIV;
		// aspetto il cambio di stato
		while(RECEIV==StatoReceiv);		//passa a space 
		// ho cambiato stato, misuro la durata
		DurataI=iOVF-InizioI;
		// se la durata  > IMPTOUT_TK (e lo stato era space), ho trovato un gap
		if(DurataI>IMPTOUT_TK){
			// faccio una scrittura, siccome dura un po' 
			// fermo TMR0; poi mi cerco un altro gap
			T0CONbits.TMR0ON=0;		// disabilito TMR0
			ScriviSeq(durate,idurC);
			fNuovaSeq=1;
			// decodifica la sequenza; la funzione restituisce l'ascii del tasto
			TastoRil=DecodSeq(durate,Commands);
			printf("\r\n tasto: %c \r\n",TastoRil);
			// scrive sull'LCD
			Line2LCD();
			WriteStringLCD("tasto: ");	
			WriteCharLCD(TastoRil);
			
			T0CONbits.TMR0ON=1;		// riabilito TMR0

		} else {
			//  una durata di mark o space; registro solo i mark dello Start e del Command
//			if(StatoReceiv==STATOAT){		// considero solo lo stato attivo (mark o space a seconda del telec)
				// se  lo Start, o una durata di un mark che si riferisce al Command,
				// la scrivo nell'array
//				if((idurate==0) || ( (idurate>=SCB) && (idurate<(SCB+NCB)) ) ){
					if (idurC<MAXN)	durate[idurC++]=DurataI;
//				}
				idurate++;		// incremento solo per i mark, quindi ad ogni bit (pi lo Start)
//			}	// if(StatoReceiv==STATOAT)
		}
		// ricomincio il ciclo

	}	// while(1)
	
}		//main


void ScriviSeq(unsigned char durate[],unsigned char ndurate){
	unsigned char i;
	printf(" \r\n ");	// a capo
	for(i=0;i<ndurate;i++){
		printf("%u  ",durate[i]);
	}
}
unsigned char DecodSeq(unsigned char durate[],unsigned char Commands[]){
	// durate[0] contiene lo Start; durate[1] la durata del LSB, durate[NCB] MSB
	// per ogni durate[i] confronta il valore con ONE_TK,ZERO_TK,TOL_TK per vedere se rappresenta 0 o 1
	// se no  n 0 n 1 ritorna 255
	// converte quindi durate[] binario->dec   questo  il Command
	// cerca sequenzialmente Command nell'array Commands[]
	// se lo trova restituisce l'ascii del tasto, altrimenti ritorna 255
	unsigned char Tasto=255,Command=0,i,pot2=1,ibit;
	// ciclo nell'array durate[], da 1 a NCB
	for(i=1;i<=NCB;i++){
		if( Confronta(durate[i],ZERO_TK,TOL_TK) )
			ibit=0;
		else
			if( Confronta(durate[i],ONE_TK,TOL_TK)) ibit=1; else return Tasto;
		Command+=ibit*pot2;
		pot2*=2;
	}	
	// cerca Command in Commands[]
	for(i=0;i<NTASTI;i++)
		if(Commands[i]==Command) break;
	// i primi 10 tasti sono i numeri da 0 a 9; per i successivi il carattere va cercato nell'array TastNN[];
	if(i<10) Tasto=i+48;		// codice ascii del numero
	else if(i<NTASTI) Tasto=TastiNN[i-10];
	return Tasto;
}


unsigned char Confronta(unsigned char cosa,unsigned char concosa,unsigned char Tol){
	if(  (cosa>=(concosa-Tol)) && (cosa<=(concosa+Tol)) ) return 1;
	else return 0;
}

void delays_ms(unsigned long cntr) {
	// Delay1KTCYx(1);	  ritardo di 0,2 ms con clock 20 MHz
	for (;cntr>0;cntr--)
		Delay1KTCYx(5);
}