/* 18 Telecomando IR
-- main_aTX  PIC del trasmettitore --
fotodiodo Vishay TSUS 4300, fototransistor Vishay TSOP34838 (38 kHz)

sezione TX:
fotodiodo comandato da RC1/CCP2 (attraverso un transistor
fo=38kHz ->To=26.3us  onda quadra da mandare al fotodiodo
Protocollo Sony SIRC a 12 bit
unit di tempo UT=600us
Start: 4*UT ON
Command: 7 bit
	1: 1*UT OFF, 2*UT ON
	0: 1*UT OFF, 1*UT ON 
Address: 5 bit

perci la durata : ((4+2*12)+ N_ON) * UT = (26+N_ON)*UT (min 15.6ms, max 22.8ms)
il segnale viene ripetuto ogni 45ms (start to start) finch il tasto  premuto.
uso TMR1 per avere un'interruzione ogni UT=600us. 	
 con FOSC=20 MHz, T1_PS=1 => T1=0.2 microsecondi, overflow ogni 2^16 * 0.2=13.107 ms 
 per avere un'int dopo DT us devo preimpostare TMR1 in modo che (2^16-baseT1)*T1=DT,
  => baseT1=2^16-DT/T1  =65535-5*DT  con T1_PS=1 (baseT1=65035 per 100us, 62535 per DT=600us)
oppure (faccio cos):
	uso TMR0 a 8 bit (TM1  solo a 16 bit), T0_PS=1, ovf ogni 255*0.2us =51us;
	con baseT1=5 => DT=250*0.2= 50us
   
*/
#pragma config FOSC=HS,LVP=OFF,WDT=OFF
#pragma config PWRT = ON,PBADEN=OFF
#pragma config CCP2MX = ON		//modulo CCP2 su RC1

#include <p18f4550.h>
// #include <stdio.h>
#include <pwm.h>		// libreria C18 per PWM
#include <timers.h>
// usa Delay1KTCYx(n) ritardo (0,2 x n) ms, Delay10KTCYx(n) ritardo (2 x n) ms
#include <delays.h>	

void delays_ms(unsigned long);	//richiamata in LCD_44780.c e keyb.c
//#define LCD_DEFAULT
//#include "LCD_44780.c"
#include "keyb.c"


#define FOTOD LATCbits.LATC1	//uscita al fotodiodo RC1/CCP2
#define LED1 LATCbits.LATC0		// Led du RC0
#define NOCOMMAND 255
#define DCY 500		// duty cycle un po' meno di 0.5
#define START_US 2400
#define OFF_US 600
#define ONE_US 1200
#define ZERO_US 600


// -- VARIABILI GLOBALI
volatile unsigned char fTrasmEn=0;
volatile unsigned int iOVF=0;	// iOVF contatore di overflow di TMR1

// -- PROTOTIPI FUNZIONI
void Trasmetti(unsigned char vchr,unsigned char address);
void invia_start();
void invia_0();
void invia_1();


// -- ISR - routines interruzioni --
// prototipo funzione che gestisce l'interrupt ad alta priorit
void hInt_Event(void);

// l'interruzione fa saltare alla locazione 0x08, detta interrupt vector 
#pragma code h_int_vect=0x08
void h_int(void) {
	_asm GOTO hInt_Event _endasm
}
#pragma code

// dichiaro che la funzione di gestione interrupt si chiama hInt_Event
#pragma interrupt hInt_Event

// definizione della funzione di gestione int (ad alta priorit, modo compatibile)
void hInt_Event(void) {
	if (INTCONbits.TMR0IF == 1) {	//  avvenuto un overflow in TMR1
		INTCONbits.TMR0IF = 0;	// resetto l'IF di TMR1
		// conteggio overflow; ogni ovf vale 50us
		iOVF++;
		if (iOVF==900){
			fTrasmEn=1;		// riabilita la trasmissione dopo 45ms
			LED1=!LED1;		// fa lampeggiare LED1
			// iOVF=0;		// DEBUG
		}
		WriteTimer0(5); // per avere DT=(255-5)*0.2= 50us
	}
}
// -- fine ISR --	


void main(void) {

	unsigned char period,chr,address=0;
	unsigned char command=NOCOMMAND;	// codice da trasmettere; 
	unsigned char mask,i,iTrasm=0;
	// impostazione porte
	ADCON1=0x0F;			// porte come digital IO

	// PORTB: usata per il tastierino
	//  RB1..RB3 input (colonne); RB4..RB7 output (righe)
	LATB=0xFF;	// inizialmente righe alte perch ingressi collegati a R di pullup
	INTCON2bits.RBPU=0;		// abilita le R di pull-up sulla PORTB
	TRISB=0b00001111;	

	LATC=0;
	//TRISC=0b11111001;
	TRISC=0; // RC1/CCP2 output
	

	// Apro il TMR2 per il PWM
 	OpenTimer2( TIMER_INT_OFF & T2_PS_1_1 & T2_POST_1_1); 

	/* Imposto il modulo PWM
	per avere una frequenza di 38 kHz:
	period=Tpwm*Fosc/(4*T2_PS)-1   = 26.3us*20MHz/(4*1)=131.5
	*/
	period=131;
 	OpenPWM2( period ); 
	// Imposta TMR1
	//	con FOSC=20 MHz, T1_PS=8 => T1=1.6 microsecondi 
    //	a 16 bit T1 va in overflow (interruzione) ogni 2^16 * 1,6us= 0.1 s

	// TMR0 a 8 bit, T1_PS=1, ovf ogni 255*0.2us =51us;
	OpenTimer0(TIMER_INT_ON & T0_SOURCE_INT & T0_PS_1_1 & T0_8BIT );
	 	
	// abilito le interruzioni
	RCONbits.IPEN=0;	// modalit compatibile PIC16
	INTCONbits.GIE=1;	// abilito l'int globale
	INTCONbits.PEIE=1;	// abilito l'int periferiche

	KEYB_INIT(); // Inizializza la tastiera a matrice con il layout di base.					  
//	OpenLCD(20);	// inizializzazione LCD
// 	WriteStringLCD ("OK");
	
 	while (1) {
		// se  in corso una serie di 3 trasmissioni
		if(fTrasmEn && iTrasm<3 && command!=NOCOMMAND) {
			// esegui una trasmissione
			//disabilita una successiva trasmissione; verr riabilitata dalla routine 
			// int di TMR1 allo scadere dei 45ms (900 interruzioni) dall'inizio di questa
			fTrasmEn=0;
			iOVF=0;		// resetta il contatore tempi
			Trasmetti(command,address);
			iTrasm++;	// incrementa il contatore trasmissioni
			if(iTrasm==3) {
				// set di 3 trasmissioni concluso; predispone e aspetta un nuovo tasto 
				iTrasm=0;
				command=NOCOMMAND;
			}	
		}
		else {
			// legge il tastierino 
			chr = KEYB_READ(OPEN);
			// se  stato premuto un tasto
			if (chr!= NOTASTO) {
				// accende il led su RA0 (test keypad)
				if (chr=='1') LATAbits.LATA0=1; else LATAbits.LATA0=0; 
				if (chr=='2') LATAbits.LATA1=1; else LATAbits.LATA1=0;
				switch (chr) {
					case '0':
						command=9;	
					break;						
					case '1':
					case '2':
					case '3':
					case '4':
					case '5':
					case '6':
					case '7':
					case '8':
					case '9':
						//'1' corrisponde a codice di comando 0
						//l'ascii di '1'  49 ; altri tasti a seguire
						command=chr-49;	
					break;
				} // switch (chr)
			}	//if (chr!= NOTASTO)
		}	// if(fTrasmEn && iTrasm<3)

	}	// while (1)
	
} //main(

void invia_start() {
	// segnale on per START_US=2400us
	SetDCPWM2 ( DCY);
	// aspetto START_US=2400us
	// Delay100TCYx(1) ritardo di 20us
	Delay100TCYx(120);		// 120=START_US/20
	SetDCPWM2(0);	// duty cycle 0
}

void invia_0() {
	// segnale off per OFF_US=600us, quindi on per ZERO_US=600us
	SetDCPWM2(0);	// duty cycle 0
	Delay100TCYx(30);		// 30=OFF_US/20
	SetDCPWM2(DCY);		// mark
	Delay100TCYx(30);		// 30=ZERO_US/20
	SetDCPWM2(0);	// ritorna basso
}

void invia_1() {
// segnale off per OFF_US=600us, quindi on per ONE_US=1200us
	SetDCPWM2(0);	// duty cycle 0
	Delay100TCYx(30);		// 30=OFF_US/20
	SetDCPWM2(DCY);	// duty cycle 0
	Delay100TCYx(60);		// 60=OFF_US/20
	SetDCPWM2(0);	// ritorna basso
}

void Trasmetti(unsigned char command,unsigned char address) {
		unsigned char i,mask;
		invia_start();
		// invia il Command di 7 bit
		// scorre vchr (binario) da LSB per 7 bit
		mask=1;
		i=0;
		for(i=0;i<7;i++) {
			if(command & mask) invia_1();
			else invia_0();
			mask<<=1;	
		}
		// invia l'Address di 5 bit
		mask=1;
		i=0;
		for(i=0;i<5;i++) {
			if(address & mask) invia_1();
			else invia_0();
			mask<<=1;	
		}
}

void delays_ms(unsigned long cntr) {
	// Delay1KTCYx(1);	  ritardo di 0,2 ms con clock 20 MHz
	for (;cntr>0;cntr--)
		Delay1KTCYx(5);
}