// telec.c

// richiede la libreria  "LCD_44780.c"

// -- INCLUDE --
/* "telec.h" :specifiche del telecomando
	definisce le costanti: 
	SIGLATELEC	// sigla del telecomando
	NTASTI 		//n. tasti da rilevare
	STATOAT 	// stato attivo 0=mark, 1=space
	ONE_TK 		// durata dello stato attivo dell'ONE in ticks di 50us	
	ZERO_TK
	IMPTOUT_TK 	// timeout= durata massima impulso= durata minima gap (10ms)
	TOL_TK 		// margine di tolleranza in + e -
	SCB 1		// StartCommandBit:dove comincia il command nella squenza impulsi (lo Start  0) 
	NCB 7		//n. di bit del command
	e gli array:
	unsigned char Commands[]	//array dei Commands per i tasti
	unsigned char TastiNN[]		//array dei codici ascii dei tasti non numerici
*/
#include "telec.h"

// -- PROTOTIPI FUNZIONI locali alla libreria --
void ScriviSeq(unsigned char durate[],unsigned char idurate);
unsigned char DecodSeq(unsigned char durate[],unsigned char Commands[]);
unsigned char Confronta(unsigned char cosa,unsigned char concosa,unsigned char Tol);

void AscoltaRicev(void){
	unsigned int InizioI,FineI,DurataI;	
	unsigned char i,fGap=0,StatoReceiv,TastoRil;

	// se  una nuova sequenza
	if(fNuovaSeq) {
		fGap=0;		// devo cercare un gap
		// azzero gli array durate e stati
		for(i=0;i<MAXN;i++) {
			durate[i]=0;	
		//	StatiRec[i]=2;	// 2 = stato non valido
		}
		idurate=0;		// indice nell'array durate
		idurC=0;		// indice durate registrate (Start + mark dei Commands)
		// aspetto un mark successivo a un gap (space di durata>IMPTOUT_TK)		
		// continuo a misurare gli space finch non trovo un gap
		while(!fGap){
			if(RECEIV==SPACE){		// sono in space
				iOVF=0;
				InizioI=iOVF;
				while(RECEIV==SPACE);	//finch non passa a mark
				DurataI=iOVF-InizioI;
				if(DurataI>IMPTOUT_TK) fGap=1;
			}
		}	//while(!fGap)
		// finalmente ho trovato un gap; ora sono all'inizio del primo mark
		StatoReceiv=RECEIV;	// mark  (space)
		fNuovaSeq=0;
	}	// if(fNuovaSeq)

	// continuo a misurare gli impulsi finch non trovo un altro gap;
	iOVF=0;		// azzero la base tempi 
	InizioI=iOVF;
	StatoReceiv=RECEIV;
	// aspetto il cambio di stato
	while(RECEIV==StatoReceiv);		//passa a space 
	// ho cambiato stato, misuro la durata
	DurataI=iOVF-InizioI;
	// se la durata  > IMPTOUT_TK (e lo stato era space), ho trovato un gap
	if(DurataI>IMPTOUT_TK){
		fNuovaSeq=1;
		// se sono stati ricevuti tutti gli NCB bit del Command faccio una scrittura;
		if(idurC>=NCB) {
			// siccome dura un po'fermo TMR0;
			T0CONbits.TMR0ON=0;		// disabilito TMR0
			ScriviSeq(durate,idurC);
			// decodifica la sequenza; la funzione restituisce l'ascii del tasto
			TastoRil=DecodSeq(durate,Commands);
			printf("\r\n tasto: %c \r\n",TastoRil);
			// scrive sull'LCD
			Line2LCD();
			WriteStringLCD("tasto: ");	
			WriteCharLCD(TastoRil);
			
			T0CONbits.TMR0ON=1;		// riabilito TMR0
		}
	} else {
		//  una durata di mark o space; registro solo i mark dello Start e del Command
		if(StatoReceiv==STATOAT){		// considero solo lo stato attivo (mark o space a seconda del telec)
			// se  lo Start, o una durata di un mark che si riferisce al Command,
			// la scrivo nell'array
			if((idurate==0) || ( (idurate>=SCB) && (idurate<(SCB+NCB)) ) ){
				if (idurC<MAXN)	durate[idurC++]=DurataI;
			}
			idurate++;		// incremento solo per i mark, quindi ad ogni bit (pi lo Start)
		}	// if(StatoReceiv==MARK)
	}
	// ricomincio il ciclo

}	// AscoltaRicev()

void ScriviSeq(unsigned char durate[],unsigned char ndurate){
	unsigned char i;
	printf(" \r\n ");	// a capo
	for(i=0;i<ndurate;i++){
		printf("%u  ",durate[i]);
	}
}
unsigned char DecodSeq(unsigned char durate[],unsigned char Commands[]){
	// durate[0] contiene lo Start; durate[1] la durata del LSB, durate[NCB] MSB
	// per ogni durate[i] confronta il valore con ONE_TK,ZERO_TK,TOL_TK per vedere se rappresenta 0 o 1
	// se no  n 0 n 1 ritorna 255
	// converte quindi durate[] binario->dec   questo  il Command
	// cerca sequenzialmente Command nell'array Commands[]
	// se lo trova restituisce l'ascii del tasto, altrimenti ritorna 255
	unsigned char Tasto=255,Command=0,i,pot2=1,ibit;
	// ciclo nell'array durate[], da 1 a NCB
	for(i=1;i<=NCB;i++){
		if( Confronta(durate[i],ZERO_TK,TOL_TK) )
			ibit=0;
		else
			if( Confronta(durate[i],ONE_TK,TOL_TK)) ibit=1; else return Tasto;
		Command+=ibit*pot2;
		pot2*=2;
	}	
	// cerca Command in Commands[]
	for(i=0;i<NTASTI;i++)
		if(Commands[i]==Command) break;
	// i primi 10 tasti sono i numeri da 0 a 9; per i successivi il carattere va cercato nell'array TastNN[];
	if(i<10) Tasto=i+48;		// codice ascii del numero
	else if(i<NTASTI) Tasto=TastiNN[i-10];
	return Tasto;
}


unsigned char Confronta(unsigned char cosa,unsigned char concosa,unsigned char Tol){
	if(  (cosa>=(concosa-Tol)) && (cosa<=(concosa+Tol)) ) return 1;
	else return 0;
}
